@@ -8,7 +8,7 @@ def test_llama_cpp_tokenization():
     llama = llama_cpp.Llama(model_path=MODEL, vocab_only=True, verbose=False)
 
     assert llama
-    assert llama.ctx is not None
+    assert llama._ctx.ctx is not None
 
     text = b"Hello World"
 
@@ -37,7 +37,7 @@ def test_llama_cpp_tokenization():
 
 def test_llama_patch(monkeypatch):
     llama = llama_cpp.Llama(model_path=MODEL, vocab_only=True)
-    n_vocab = llama_cpp.llama_n_vocab(llama.model)
+    n_vocab = llama_cpp.llama_n_vocab(llama._model.model)
 
     ## Set up mock function
     def mock_eval(*args, **kwargs):
@@ -48,7 +48,7 @@ def test_llama_patch(monkeypatch):
             *[llama_cpp.c_float(0) for _ in range(n_vocab)]
         )
 
-    monkeypatch.setattr("llama_cpp.llama_cpp.llama_eval", mock_eval)
+    monkeypatch.setattr("llama_cpp.llama_cpp.llama_decode", mock_eval)
     monkeypatch.setattr("llama_cpp.llama_cpp.llama_get_logits", mock_get_logits)
 
     output_text = " jumps over the lazy dog."
@@ -138,7 +138,7 @@ def test_utf8(monkeypatch):
             *[llama_cpp.c_float(0) for _ in range(n_vocab)]
         )
 
-    monkeypatch.setattr("llama_cpp.llama_cpp.llama_eval", mock_eval)
+    monkeypatch.setattr("llama_cpp.llama_cpp.llama_decode", mock_eval)
     monkeypatch.setattr("llama_cpp.llama_cpp.llama_get_logits", mock_get_logits)
 
     output_text = "😀"
